@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,18 +18,20 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: os400sys.c,v 1.3 2008-01-16 16:04:47 patrickm Exp $
+ * $Id: os400sys.c,v 1.7 2009-05-02 02:37:34 yangtse Exp $
  *
  ***************************************************************************/
 
 /* OS/400 additional support. */
 
-#include "config-os400.h"	/* Not setup.h: we only need some defines. */
+#include "config-os400.h"       /* Not setup.h: we only need some defines. */
 
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 
 #include <stdlib.h>
+#include <stddef.h>
 #include <string.h>
 #include <pthread.h>
 #include <netdb.h>
@@ -190,7 +192,7 @@ buffer_threaded(localkey_t key, long size)
 
     /* Allocate buffer descriptors for the current thread. */
 
-    if (!(bufs = (buffer_t *) calloc((size_t) LK_LAST, sizeof *bufs)))
+    if (!(bufs = calloc((size_t) LK_LAST, sizeof *bufs)))
       return (char *) NULL;
 
     if (pthread_setspecific(thdkey, (void *) bufs)) {
@@ -218,7 +220,7 @@ buffer_undef(localkey_t key, long size)
   if (Curl_thread_buffer == buffer_undef) {     /* If unchanged during lock. */
     if (!pthread_key_create(&thdkey, thdbufdestroy))
       Curl_thread_buffer = buffer_threaded;
-    else if (!(locbufs = (buffer_t *) calloc((size_t) LK_LAST,
+    else if (!(locbufs = calloc((size_t) LK_LAST,
                                              sizeof *locbufs))) {
       pthread_mutex_unlock(&mutex);
       return (char *) NULL;
@@ -235,9 +237,9 @@ buffer_undef(localkey_t key, long size)
 
 
 int
-Curl_getnameinfo_a(const struct sockaddr * sa, socklen_t salen,
-              char * nodename, socklen_t nodenamelen,
-              char * servname, socklen_t servnamelen,
+Curl_getnameinfo_a(const struct sockaddr * sa, curl_socklen_t salen,
+              char * nodename, curl_socklen_t nodenamelen,
+              char * servname, curl_socklen_t servnamelen,
               int flags)
 
 {
@@ -338,37 +340,6 @@ Curl_getaddrinfo_a(const char * nodename, const char * servname,
 }
 
 
-int
-Curl_inet_ntoa_r_a(struct in_addr internet_address,
-                   char * output_buffer, int output_buffer_length)
-
-{
-  int rc;
-  int i;
-  char * cp;
-
-  if (!output_buffer || output_buffer_length < 16)
-    return inet_ntoa_r(internet_address, output_buffer, output_buffer_length);
-
-  if (!(cp = malloc(output_buffer_length + 1)))
-    return -1;
-
-  rc = inet_ntoa_r(internet_address, cp, output_buffer_length);
-
-  if (rc) {
-    free(cp);
-    return rc;
-    }
-
-  cp[output_buffer_length - 1] = '\0';
-  i = strlen(cp);
-  QadrtConvertE2A(output_buffer, cp, i, i);
-  output_buffer[i] = '\0';
-  free(cp);
-  return rc;
-}
-
-
 #ifdef USE_QSOSSL
 
 /* ASCII wrappers for the SSL procedures. */
@@ -775,7 +746,7 @@ Curl_ldap_search_s_a(void * ld, char * base, int scope, char * filter,
     for (i = 0; attrs[i++];)
       ;
 
-    if (!(eattrs = (char * *) calloc(i, sizeof *eattrs)))
+    if (!(eattrs = calloc(i, sizeof *eattrs)))
       status = LDAP_NO_MEMORY;
     else {
       for (j = 0; attrs[j]; j++) {
@@ -965,3 +936,145 @@ Curl_ldap_next_attribute_a(void * ld,
 }
 
 #endif /* CURL_DISABLE_LDAP */
+
+
+static int
+convert_sockaddr(struct sockaddr_storage * dstaddr,
+                                const struct sockaddr * srcaddr, int srclen)
+
+{
+  const struct sockaddr_un * srcu;
+  struct sockaddr_un * dstu;
+  unsigned int i;
+  unsigned int dstsize;
+
+  /* Convert a socket address into job CCSID, if needed. */
+
+  if (!srcaddr || srclen < offsetof(struct sockaddr, sa_family) +
+      sizeof srcaddr->sa_family || srclen > sizeof *dstaddr) {
+    errno = EINVAL;
+    return -1;
+    }
+
+  memcpy((char *) dstaddr, (char *) srcaddr, srclen);
+
+  switch (srcaddr->sa_family) {
+
+  case AF_UNIX:
+    srcu = (const struct sockaddr_un *) srcaddr;
+    dstu = (struct sockaddr_un *) dstaddr;
+    dstsize = sizeof *dstaddr - offsetof(struct sockaddr_un, sun_path);
+    srclen -= offsetof(struct sockaddr_un, sun_path);
+    i = QadrtConvertA2E(dstu->sun_path, srcu->sun_path, dstsize - 1, srclen);
+    dstu->sun_path[i] = '\0';
+    i += offsetof(struct sockaddr_un, sun_path);
+    srclen = i;
+    }
+
+  return srclen;
+}
+
+
+int
+Curl_os400_connect(int sd, struct sockaddr * destaddr, int addrlen)
+
+{
+  int i;
+  struct sockaddr_storage laddr;
+
+  i = convert_sockaddr(&laddr, destaddr, addrlen);
+
+  if (i < 0)
+    return -1;
+
+  return connect(sd, (struct sockaddr *) &laddr, i);
+}
+
+
+int
+Curl_os400_bind(int sd, struct sockaddr * localaddr, int addrlen)
+
+{
+  int i;
+  struct sockaddr_storage laddr;
+
+  i = convert_sockaddr(&laddr, localaddr, addrlen);
+
+  if (i < 0)
+    return -1;
+
+  return bind(sd, (struct sockaddr *) &laddr, i);
+}
+
+
+int
+Curl_os400_sendto(int sd, char * buffer, int buflen, int flags,
+                                struct sockaddr * dstaddr, int addrlen)
+
+{
+  int i;
+  struct sockaddr_storage laddr;
+
+  i = convert_sockaddr(&laddr, dstaddr, addrlen);
+
+  if (i < 0)
+    return -1;
+
+  return sendto(sd, buffer, buflen, flags, (struct sockaddr *) &laddr, i);
+}
+
+
+int
+Curl_os400_recvfrom(int sd, char * buffer, int buflen, int flags,
+                                struct sockaddr * fromaddr, int * addrlen)
+
+{
+  int i;
+  int rcvlen;
+  int laddrlen;
+  const struct sockaddr_un * srcu;
+  struct sockaddr_un * dstu;
+  struct sockaddr_storage laddr;
+
+  if (!fromaddr || !addrlen || *addrlen <= 0)
+    return recvfrom(sd, buffer, buflen, flags, fromaddr, addrlen);
+
+  laddrlen = sizeof laddr;
+  laddr.ss_family = AF_UNSPEC;          /* To detect if unused. */
+  rcvlen = recvfrom(sd, buffer, buflen, flags,
+                    (struct sockaddr *) &laddr, &laddrlen);
+
+  if (rcvlen < 0)
+    return rcvlen;
+
+  switch (laddr.ss_family) {
+
+  case AF_UNIX:
+    srcu = (const struct sockaddr_un *) &laddr;
+    dstu = (struct sockaddr_un *) fromaddr;
+    i = *addrlen - offsetof(struct sockaddr_un, sun_path);
+    laddrlen -= offsetof(struct sockaddr_un, sun_path);
+    i = QadrtConvertE2A(dstu->sun_path, srcu->sun_path, i, laddrlen);
+    laddrlen = i + offsetof(struct sockaddr_un, sun_path);
+
+    if (laddrlen < *addrlen)
+      dstu->sun_path[i] = '\0';
+
+    break;
+
+  case AF_UNSPEC:
+    break;
+
+  default:
+    if (laddrlen > *addrlen)
+      laddrlen = *addrlen;
+
+    if (laddrlen)
+      memcpy((char *) fromaddr, (char *) &laddr, laddrlen);
+
+    break;
+    }
+
+  *addrlen = laddrlen;
+  return rcvlen;
+}
