@@ -5,12 +5,22 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: lib554.c,v 1.3 2008-05-22 21:49:53 danf Exp $
+ * $Id: lib554.c,v 1.6 2009-05-08 02:14:50 yangtse Exp $
  */
 
 #include "test.h"
 
-static char data[]="this is what we post to the silly web server\n";
+#include "memdebug.h"
+
+static char data[]=
+#ifdef CURL_DOES_CONVERSIONS
+  /* ASCII representation with escape sequences for non-ASCII platforms */
+  "\x74\x68\x69\x73\x20\x69\x73\x20\x77\x68\x61\x74\x20\x77\x65\x20\x70"
+  "\x6f\x73\x74\x20\x74\x6f\x20\x74\x68\x65\x20\x73\x69\x6c\x6c\x79\x20"
+  "\x77\x65\x62\x20\x73\x65\x72\x76\x65\x72\x0a";
+#else
+  "this is what we post to the silly web server\n";
+#endif
 
 struct WriteThis {
   char *readptr;
@@ -44,11 +54,14 @@ int test(char *URL)
   struct curl_httppost *lastptr=NULL;
   struct WriteThis pooh;
 
+  if (curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
+    fprintf(stderr, "curl_global_init() failed\n");
+    return TEST_ERR_MAJOR_BAD;
+  }
+
   pooh.readptr = data;
   pooh.sizeleft = strlen(data);
 
-  curl_global_init(CURL_GLOBAL_ALL);
-
   /* Fill in the file upload field */
   formrc = curl_formadd(&formpost,
                         &lastptr,
@@ -65,7 +78,14 @@ int test(char *URL)
   formrc = curl_formadd(&formpost,
                         &lastptr,
                         CURLFORM_COPYNAME, "filename",
+#ifdef CURL_DOES_CONVERSIONS
+                        /* ASCII representation with escape
+                           sequences for non-ASCII platforms */
+                        CURLFORM_COPYCONTENTS,
+                           "\x70\x6f\x73\x74\x69\x74\x32\x2e\x63",
+#else
                         CURLFORM_COPYCONTENTS, "postit2.c",
+#endif
                         CURLFORM_END);
 
   if(formrc)
@@ -75,19 +95,21 @@ int test(char *URL)
   formrc = curl_formadd(&formpost,
                         &lastptr,
                         CURLFORM_COPYNAME, "submit",
+#ifdef CURL_DOES_CONVERSIONS
+                        /* ASCII representation with escape
+                           sequences for non-ASCII platforms */
+                        CURLFORM_COPYCONTENTS, "\x73\x65\x6e\x64",
+#else
                         CURLFORM_COPYCONTENTS, "send",
+#endif
                         CURLFORM_END);
 
   if(formrc)
     printf("curl_formadd(3) = %d\n", (int)formrc);
 
-  if (curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
-    fprintf(stderr, "curl_global_init() failed\n");
-    return TEST_ERR_MAJOR_BAD;
-  }
-
   if ((curl = curl_easy_init()) == NULL) {
     fprintf(stderr, "curl_easy_init() failed\n");
+    curl_formfree(formpost);
     curl_global_cleanup();
     return TEST_ERR_MAJOR_BAD;
   }
