@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: test.h,v 1.19 2007-10-02 16:05:28 yangtse Exp $
+ * $Id: test.h,v 1.22 2009-05-01 12:39:40 yangtse Exp $
  */
 
 /* Now include the setup.h file from libcurl's private libdir (the source
@@ -15,7 +15,7 @@
 
 #include "setup.h"
 
-#include <curl.h>
+#include <curl/curl.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -33,6 +33,10 @@
 #include <unistd.h>
 #endif
 
+#ifdef TPF
+#  include "select.h"
+#endif
+
 #define TEST_ERR_MAJOR_BAD     100
 #define TEST_ERR_RUNS_FOREVER   99
 
@@ -43,8 +47,9 @@ extern char *libtest_arg3; /* set by first.c to the argv[3] or NULL */
 extern int test_argc;
 extern char **test_argv;
 
-int select_test (int num_fds, fd_set *rd, fd_set *wr, fd_set *exc,
-                 struct timeval *tv);
+extern int select_test(int num_fds, fd_set *rd, fd_set *wr, fd_set *exc,
+                       struct timeval *tv);
+
+extern int test(char *URL); /* the actual test function provided by each
+                               individual libXXX.c file */
 
-int test(char *URL); /* the actual test function provided by each individual
-                        libXXX.c file */
