@@ -6,7 +6,7 @@
 #                            | (__| |_| |  _ <| |___
 #                             \___|\___/|_| \_\_____|
 #
-# Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+# Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: runtests.pl,v 1.295 2008-05-01 10:51:16 bagder Exp $
+# $Id: runtests.pl,v 1.327 2009-05-15 11:59:24 yangtse Exp $
 ###########################################################################
 
 # Experimental hooks are available to run tests remotely on machines that
@@ -45,7 +45,7 @@
 # to do with cookies, those that set environment variables, or those that
 # do more than touch the file system in a <precheck> or <postcheck>
 # section). These can be added to the $TESTCASES line below,
-# e.g. $TESTCASES="!8 !31 !63..."
+# e.g. $TESTCASES="!8 !31 !63 !cookies..."
 #
 # Finally, to properly support -g and -n, checktestcmd needs to change
 # to check the remote system's PATH, and the places in the code where
@@ -112,6 +112,9 @@ my $SOCKSPORT; # SOCKS4/5 port
 
 my $srcdir = $ENV{'srcdir'} || '.';
 my $CURL="../src/curl"; # what curl executable to run on the tests
+my $VCURL=$CURL;   # what curl binary to use to verify the servers with
+                   # VCURL is handy to set to the system one when the one you
+                   # just built hangs or crashes and thus prevent verification
 my $DBGCURL=$CURL; #"../src/.libs/curl";  # alternative for debugging
 my $LOGDIR="log";
 my $TESTDIR="$srcdir/data";
@@ -181,6 +184,7 @@ my $has_ipv6;    # set if libcurl is built with IPv6 support
 my $has_libz;    # set if libcurl is built with libz support
 my $has_getrlimit;  # set if system has getrlimit()
 my $has_ntlm;    # set if libcurl is built with NTLM support
+my $has_charconv;# set if libcurl is built with CharConv support
 
 my $has_openssl; # built with a lib using an OpenSSL-like API
 my $has_gnutls;  # built with GnuTLS
@@ -197,6 +201,7 @@ my $skipped=0;  # number of tests skipped; reported in main loop
 my %skipped;    # skipped{reason}=counter, reasons for skip
 my @teststat;   # teststat[testnum]=reason, reasons for skip
 my %disabled_keywords;	# key words of tests to skip
+my %enabled_keywords;	# key words of tests to run
 
 my $sshdid;      # for socks server, ssh daemon version id
 my $sshdvernum;  # for socks server, ssh daemon version number
@@ -206,6 +211,8 @@ my $sshderror;   # for socks server, ssh daemon version error
 my $defserverlogslocktimeout = 20; # timeout to await server logs lock removal
 my $defpostcommanddelay = 0; # delay between command and postcheck sections
 
+my $testnumcheck; # test number, set in singletest sub.
+
 #######################################################################
 # variables the command line options may set
 #
@@ -387,7 +394,8 @@ sub checkcmd {
     my @paths=(split(":", $ENV{'PATH'}), "/usr/sbin", "/usr/local/sbin",
                "/sbin", "/usr/bin", "/usr/local/bin" );
     for(@paths) {
-        if( -x "$_/$cmd") {
+        if( -x "$_/$cmd" && ! -d "$_/$cmd") {
+            # executable bit but not a directory!
             return "$_/$cmd";
         }
     }
@@ -431,6 +439,7 @@ sub runclientoutput {
 # Memory allocation test and failure torture testing.
 #
 sub torture {
+    use POSIX "strftime";
     my $testcmd = shift;
     my $gdbline = shift;
 
@@ -467,7 +476,7 @@ sub torture {
             next;
         }
 
-        logmsg "Fail alloc no: $limit\r" if($verbose);
+        logmsg "Fail alloc no: $limit @ " . strftime ("%H:%M:%S", localtime) . "\r" if($verbose);
 
         # make the memory allocation function number $limit return failure
         $ENV{'CURL_MEMLIMIT'} = $limit;
@@ -484,6 +493,7 @@ sub torture {
         else {
             $ret = runclient($testcmd);
         }
+        #logmsg "$_ Returned " . $ret / 256 . "\n";
 
         # Now clear the variable again
         $ENV{'CURL_MEMLIMIT'} = undef;
@@ -583,7 +593,7 @@ sub stopserver {
 
 sub verifyhttp {
     my ($proto, $ip, $port) = @_;
-    my $cmd = "$CURL -m$server_response_maxtime -o log/verifiedserver -ksvg \"$proto://$ip:$port/verifiedserver\" 2>log/verifyhttp";
+    my $cmd = "$VCURL --max-time $server_response_maxtime --output $LOGDIR/verifiedserver --insecure --silent --verbose --globoff \"$proto://$ip:$port/verifiedserver\" 2>$LOGDIR/verifyhttp";
     my $pid;
 
     # verify if our/any server is running on this port
@@ -594,7 +604,7 @@ sub verifyhttp {
     my $data;
 
     if($res && $verbose) {
-        open(ERR, "<log/verifyhttp");
+        open(ERR, "<$LOGDIR/verifyhttp");
         my @e = <ERR>;
         close(ERR);
         logmsg "RUN: curl command returned $res\n";
@@ -604,7 +614,7 @@ sub verifyhttp {
             }
         }
     }
-    open(FILE, "<log/verifiedserver");
+    open(FILE, "<$LOGDIR/verifiedserver");
     my @file=<FILE>;
     close(FILE);
     $data=$file[0]; # first line
@@ -636,9 +646,9 @@ sub verifyftp {
     my $time=time();
     my $extra;
     if($proto eq "ftps") {
-    	$extra = "-k --ftp-ssl-control ";
+    	$extra = "--insecure --ftp-ssl-control ";
     }
-    my $cmd="$CURL -m$server_response_maxtime --silent -vg $extra\"$proto://$ip:$port/verifiedserver\" 2>log/verifyftp";
+    my $cmd="$VCURL --max-time $server_response_maxtime --silent --verbose --globoff $extra\"$proto://$ip:$port/verifiedserver\" 2>$LOGDIR/verifyftp";
     # check if this is our server running on this port:
     my @data=runclientoutput($cmd);
     logmsg "RUN: $cmd\n" if($verbose);
@@ -838,6 +848,7 @@ sub runhttpserver {
         # it is NOT alive
         logmsg "RUN: failed to start the HTTP$nameext server\n";
         stopserver("$pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -848,6 +859,7 @@ sub runhttpserver {
         logmsg "RUN: HTTP$nameext server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$httppid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -902,6 +914,7 @@ sub runhttpsserver {
         # it is NOT alive
         logmsg "RUN: failed to start the HTTPS server\n";
         stopservers($verbose);
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return(0,0);
     }
@@ -912,6 +925,7 @@ sub runhttpsserver {
         logmsg "RUN: HTTPS server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$httpspid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -980,6 +994,7 @@ sub runftpserver {
         # it is NOT alive
         logmsg "RUN: failed to start the FTP$id$nameext server\n";
         stopserver("$pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -990,6 +1005,7 @@ sub runftpserver {
         logmsg "RUN: FTP$id$nameext server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$ftppid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -1044,6 +1060,7 @@ sub runftpsserver {
         # it is NOT alive
         logmsg "RUN: failed to start the FTPS server\n";
         stopservers($verbose);
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return(0,0);
     }
@@ -1054,6 +1071,7 @@ sub runftpsserver {
         logmsg "RUN: FTPS server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$ftpspid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -1118,6 +1136,7 @@ sub runtftpserver {
         # it is NOT alive
         logmsg "RUN: failed to start the TFTP$id$nameext server\n";
         stopserver("$pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -1128,6 +1147,7 @@ sub runtftpserver {
         logmsg "RUN: TFTP$id$nameext server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$tftppid $pid2");
+        displaylogs($testnumcheck);
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
@@ -1538,7 +1558,7 @@ sub checksystem {
                 # debug is a listed "feature", use that knowledge
                 $curl_debug = 1;
                 # set the NETRC debug env
-                $ENV{'CURL_DEBUG_NETRC'} = 'log/netrc';
+                $ENV{'CURL_DEBUG_NETRC'} = "$LOGDIR/netrc";
             }
             if($feat =~ /SSL/i) {
                 # ssl enabled
@@ -1562,6 +1582,10 @@ sub checksystem {
                 # NTLM enabled
                 $has_ntlm=1;
             }
+            if($feat =~ /CharConv/i) {
+                # CharConv enabled
+                $has_charconv=1;
+            }
         }
     }
     if(!$curl) {
@@ -1728,6 +1752,10 @@ sub singletest {
     my %feature;
     my $cmd;
 
+    # copy test number to a global scope var, this allows
+    # testnum checking when starting test harness servers.
+    $testnumcheck = $testnum;
+
     # load the test case file definition
     if(loadtest("${TESTDIR}/test${testnum}")) {
         if($verbose) {
@@ -1820,13 +1848,20 @@ sub singletest {
 
     if(!$why) {
         my @keywords = getpart("info", "keywords");
+	my $match;
         my $k;
         for $k (@keywords) {
             chomp $k;
             if ($disabled_keywords{$k}) {
             	$why = "disabled by keyword";
+            } elsif ($enabled_keywords{$k}) {
+            	$match = 1;
             }
         }
+
+	if(!$why && !$match && %enabled_keywords) {
+	  $why = "disabled by missing keyword";
+	}
     }
 
     if(!$why) {
@@ -1843,6 +1878,8 @@ sub singletest {
             if($o[0]) {
                 $why = $o[0];
                 chomp $why;
+            } elsif($?) {
+                $why = "precheck command error";
             }
             logmsg "prechecked $cmd\n" if($verbose);
         }
@@ -2008,8 +2045,8 @@ sub singletest {
 
     my $cmdargs;
     if(!$tool) {
-        # run curl, add -v for debug information output
-        $cmdargs ="$out --include -v --trace-time $cmd";
+        # run curl, add --verbose for debug information output
+        $cmdargs ="$out --include --verbose --trace-time $cmd";
     }
     else {
         $cmdargs = " $cmd"; # $cmd is the command line for the test file
@@ -2040,7 +2077,7 @@ sub singletest {
 
     my $usevalgrind = $valgrind && ((getpart("verify", "valgrind"))[0] !~ /disable/);
     if($usevalgrind) {
-        $CMDLINE = "$valgrind ".$valgrind_tool."--leak-check=yes --num-callers=16 ${valgrind_logfile}=log/valgrind$testnum $CMDLINE";
+        $CMDLINE = "$valgrind ".$valgrind_tool."--leak-check=yes --num-callers=16 ${valgrind_logfile}=$LOGDIR/valgrind$testnum $CMDLINE";
     }
 
     $CMDLINE .= "$cmdargs >>$STDOUT 2>>$STDERR";
@@ -2071,18 +2108,20 @@ sub singletest {
     }
 
     if($gdbthis) {
-        open(GDBCMD, ">log/gdbcmd");
+        my $gdbinit = "$TESTDIR/gdbinit$testnum";
+        open(GDBCMD, ">$LOGDIR/gdbcmd");
         print GDBCMD "set args $cmdargs\n";
         print GDBCMD "show args\n";
+        print GDBCMD "source $gdbinit\n" if -e $gdbinit;
         close(GDBCMD);
     }
     # run the command line we built
     if ($torture) {
         $cmdres = torture($CMDLINE,
-                       "$gdb --directory libtest $DBGCURL -x log/gdbcmd");
+                       "$gdb --directory libtest $DBGCURL -x $LOGDIR/gdbcmd");
     }
     elsif($gdbthis) {
-        runclient("$gdb --directory libtest $DBGCURL -x log/gdbcmd");
+        runclient("$gdb --directory libtest $DBGCURL -x $LOGDIR/gdbcmd");
         $cmdres=0; # makes it always continue after a debugged run
     }
     else {
@@ -2094,7 +2133,8 @@ sub singletest {
             $cmdres = 1000;
         }
         else {
-            $cmdres /= 256;
+            $cmdres >>= 8;
+            $cmdres = (2000 + $signal_num) if($signal_num && !$cmdres);
         }
     }
     if(!$dumped_core) {
@@ -2108,11 +2148,11 @@ sub singletest {
         logmsg "core dumped\n";
         if(0 && $gdb) {
             logmsg "running gdb for post-mortem analysis:\n";
-            open(GDBCMD, ">log/gdbcmd2");
+            open(GDBCMD, ">$LOGDIR/gdbcmd2");
             print GDBCMD "bt\n";
             close(GDBCMD);
-            runclient("$gdb --directory libtest -x log/gdbcmd2 -batch $DBGCURL core ");
-     #       unlink("log/gdbcmd2");
+            runclient("$gdb --directory libtest -x $LOGDIR/gdbcmd2 -batch $DBGCURL core ");
+     #       unlink("$LOGDIR/gdbcmd2");
         }
     }
 
@@ -2351,7 +2391,8 @@ sub singletest {
     }
     else {
         if(!$short) {
-            printf "\ncurl returned $cmdres, %d was expected\n", $errorcode;
+            printf("\n%s returned $cmdres, %d was expected\n",
+                   (!$tool)?"curl":$tool, $errorcode);
         }
         logmsg " exit FAILED\n";
         return 1;
@@ -2423,7 +2464,7 @@ sub singletest {
             if(!$src) {
                 $src=".";
             }
-            my @e = valgrindparse($src, $feature{'SSL'}, "log/$l");
+            my @e = valgrindparse($src, $feature{'SSL'}, "$LOGDIR/$l");
             if($e[0]) {
                 logmsg " valgrind ERROR ";
                 logmsg @e;
@@ -2481,6 +2522,7 @@ sub stopservers {
             }
             $prev = $pid;
         }
+        delete $run{$server};
     }
     ftpkillslaves($verbose);
 }
@@ -2698,7 +2740,7 @@ sub serverfortest {
 	chomp $proto;
 	if (! grep /^$proto$/, @protocols) {
 	    if (substr($proto,0,5) ne "socks") {
-		    return "curl lacks any $proto support";
+		    return "curl lacks $proto support";
 	    }
 	}
     }
@@ -2778,10 +2820,10 @@ while(@ARGV) {
         # keep stdout and stderr files after tests
         $keepoutfiles=1;
     }
-    elsif($ARGV[0] eq "-h") {
+    elsif(($ARGV[0] eq "-h") || ($ARGV[0] eq "--help")) {
         # show help text
         print <<EOHELP
-Usage: runtests.pl [options] [test number(s)]
+Usage: runtests.pl [options] [test selection(s)]
   -a       continue even if a test fails
   -bN      use base port number N for test servers (default $base)
   -c path  use this curl executable
@@ -2797,7 +2839,8 @@ Usage: runtests.pl [options] [test number(s)]
   -v       verbose output
   [num]    like "5 6 9" or " 5 to 22 " to run those tests only
   [!num]   like "!5 !6 !9" to disable those tests
-  [!keyword] like "!cookies !IPv6" to disable tests with those key words
+  [keyword] like "IPv6" to select only tests containing the key word
+  [!keyword] like "!cookies" to disable any tests containing the key word
 EOHELP
     ;
         exit;
@@ -2824,6 +2867,9 @@ EOHELP
     elsif($ARGV[0] =~ /^!(.+)/) {
         $disabled_keywords{$1}=$1;
     }
+    elsif($ARGV[0] =~ /^([-[{a-zA-Z].*)/) {
+        $enabled_keywords{$1}=$1;
+    }
     else {
     	print "Unknown option: $ARGV[0]\n";
     	exit;
@@ -2940,7 +2986,7 @@ if ( $TESTCASES eq "all") {
     for(@cmds) {
         $_ =~ s/[a-z\/\.]*//g;
     }
-    # the the numbers from low to high
+    # sort the numbers from low to high
     foreach my $n (sort { $a <=> $b } @cmds) {
         if($disabled{$n}) {
             # skip disabled test cases
@@ -2963,11 +3009,10 @@ open(CMDLOG, ">$CURLLOG") ||
 #######################################################################
 
 # Display the contents of the given file.  Line endings are canonicalized
-# and excessively long files are truncated
+# and excessively long files are elided
 sub displaylogcontent {
     my ($file)=@_;
     if(open(SINGLE, "<$file")) {
-        my $lfcount;
         my $linecount = 0;
         my $truncate;
         my @tail;
@@ -2975,29 +3020,17 @@ sub displaylogcontent {
             $string =~ s/\r\n/\n/g;
             $string =~ s/[\r\f\032]/\n/g;
             $string .= "\n" unless ($string =~ /\n$/);
-            $lfcount = $string =~ tr/\n//;
-            if($lfcount == 1) {
-                $string =~ s/\n//;
-                $string =~ s/\s*\!$//;
-                $linecount++;
-                if ($truncate) {
-                    push @tail, " $string\n";
-                } else {
-                    logmsg " $string\n";
-                }
-            }
-            else {
-                for my $line (split("\n", $string)) {
-                    $line =~ s/\s*\!$//;
-                    $linecount++;
-                    if ($truncate) {
-                        push @tail, " $line\n";
-                    } else {
-                        logmsg " $line\n";
-		    }
-                }
-            }
-            $truncate = $linecount > 1000;
+            $string =~ tr/\n//;
+	    for my $line (split("\n", $string)) {
+		$line =~ s/\s*\!$//;
+		if ($truncate) {
+		    push @tail, " $line\n";
+		} else {
+		    logmsg " $line\n";
+		}
+		$linecount++;
+		$truncate = $linecount > 1000;
+	    }
         }
         if (@tail) {
             logmsg "=== File too long: lines here were removed\n";
@@ -3015,7 +3048,7 @@ sub displaylogs {
     my @logs = readdir(DIR);
     closedir(DIR);
 
-    logmsg "== Contents of files in the log/ dir after test $testnum\n";
+    logmsg "== Contents of files in the $LOGDIR/ dir after test $testnum\n";
     foreach my $log (sort @logs) {
         if($log =~ /\.(\.|)$/) {
             next; # skip "." and ".."
