@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sockfilt.c,v 1.55 2008-10-01 17:34:25 danf Exp $
+ * $Id: sockfilt.c,v 1.63 2009-05-18 02:19:21 yangtse Exp $
  ***************************************************************************/
 
 /* Purpose
@@ -94,8 +94,7 @@
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#ifdef _XOPEN_SOURCE_EXTENDED
-/* This define is "almost" required to build on HPUX 11 */
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
 #ifdef HAVE_NETDB_H
@@ -146,11 +145,25 @@ enum sockmode {
 
 typedef RETSIGTYPE (*SIGHANDLER_T)(int);
 
+#ifdef SIGHUP
 static SIGHANDLER_T old_sighup_handler  = SIG_ERR;
+#endif
+
+#ifdef SIGPIPE
 static SIGHANDLER_T old_sigpipe_handler = SIG_ERR;
+#endif
+
+#ifdef SIGALRM
 static SIGHANDLER_T old_sigalrm_handler = SIG_ERR;
+#endif
+
+#ifdef SIGINT
 static SIGHANDLER_T old_sigint_handler  = SIG_ERR;
+#endif
+
+#ifdef SIGTERM
 static SIGHANDLER_T old_sigterm_handler = SIG_ERR;
+#endif
 
 /* var which if set indicates that the program should finish execution */
 
@@ -545,9 +558,9 @@ static bool juggle(curl_socket_t *sockfdp,
     else if(!memcmp("PORT", buffer, 4)) {
       /* Question asking us what PORT number we are listening to.
          Replies to PORT with "IPv[num]/[port]" */
-      sprintf((char *)buffer, "%s/%d\n", ipv_inuse, (int)port);
+      sprintf((char *)buffer, "%s/%hu\n", ipv_inuse, port);
       buffer_len = (ssize_t)strlen((char *)buffer);
-      snprintf(data, sizeof(data), "PORT\n%04x\n", buffer_len);
+      snprintf(data, sizeof(data), "PORT\n%04zx\n", buffer_len);
       if(!write_stdout(data, 10))
         return FALSE;
       if(!write_stdout(buffer, buffer_len))
@@ -647,7 +660,7 @@ static bool juggle(curl_socket_t *sockfdp,
       return TRUE;
     }
 
-    snprintf(data, sizeof(data), "DATA\n%04x\n", nread_socket);
+    snprintf(data, sizeof(data), "DATA\n%04zx\n", nread_socket);
     if(!write_stdout(data, 10))
       return FALSE;
     if(!write_stdout(buffer, nread_socket))
@@ -668,7 +681,7 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
 #ifdef ENABLE_IPV6
   struct sockaddr_in6 me6;
 #endif /* ENABLE_IPV6 */
-  int flag = 1;
+  int flag;
   int rc;
   int totdelay = 0;
   int maxretr = 10;
@@ -678,16 +691,20 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
 
   do {
     attempt++;
+    flag = 1;
     rc = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
          (void *)&flag, sizeof(flag));
     if(rc) {
       error = SOCKERRNO;
+      logmsg("setsockopt(SO_REUSEADDR) failed with error: (%d) %s",
+             error, strerror(error));
       if(maxretr) {
         rc = wait_ms(delay);
         if(rc) {
           /* should not happen */
           error = SOCKERRNO;
-          logmsg("wait_ms() failed: (%d) %s", error, strerror(error));
+          logmsg("wait_ms() failed with error: (%d) %s",
+                 error, strerror(error));
           sclose(sock);
           return CURL_SOCKET_BAD;
         }
@@ -708,6 +725,9 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
     logmsg("Continuing anyway...");
   }
 
+  /* When the specified listener port is zero, it is actually a
+     request to let the system choose a non-zero available port. */
+
 #ifdef ENABLE_IPV6
   if(!use_ipv6) {
 #endif
@@ -734,21 +754,53 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
   }
 
   if(!*listenport) {
-    /* The system picked a port number, now figure out which port we actually
-       got */
-    /* we succeeded to bind */
-    struct sockaddr_in add;
-    socklen_t socksize = sizeof(add);
-
-    if(getsockname(sock, (struct sockaddr *) &add,
-                   &socksize)<0) {
+    /* The system was supposed to choose a port number, figure out which
+       port we actually got and update the listener port value with it. */
+    curl_socklen_t la_size;
+    struct sockaddr *localaddr;
+    struct sockaddr_in localaddr4;
+#ifdef ENABLE_IPV6
+    struct sockaddr_in6 localaddr6;
+    if(!use_ipv6) {
+#endif
+      la_size = sizeof(localaddr4);
+      localaddr = (struct sockaddr *)&localaddr4;
+#ifdef ENABLE_IPV6
+    }
+    else {
+      la_size = sizeof(localaddr6);
+      localaddr = (struct sockaddr *)&localaddr6;
+    }
+#endif
+    memset(localaddr, 0, (size_t)la_size);
+    if(getsockname(sock, localaddr, &la_size) < 0) {
       error = SOCKERRNO;
       logmsg("getsockname() failed with error: (%d) %s",
              error, strerror(error));
       sclose(sock);
       return CURL_SOCKET_BAD;
     }
-    *listenport = ntohs(add.sin_port);
+    switch (localaddr->sa_family) {
+    case AF_INET:
+      *listenport = ntohs(localaddr4.sin_port);
+      break;
+#ifdef ENABLE_IPV6
+    case AF_INET6:
+      *listenport = ntohs(localaddr6.sin6_port);
+      break;
+#endif
+    default:
+      break;
+    }
+    if(!*listenport) {
+      /* Real failure, listener port shall not be zero beyond this point. */
+      logmsg("Apparently getsockname() succeeded, with listener port zero.");
+      logmsg("A valid reason for this failure is a binary built without");
+      logmsg("proper network library linkage. This might not be the only");
+      logmsg("reason, but double check it before anything else.");
+      sclose(sock);
+      return CURL_SOCKET_BAD;
+    }
   }
 
   /* start accepting connections */
