@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sws.c,v 1.120 2008-04-23 23:55:34 yangtse Exp $
+ * $Id: sws.c,v 1.131 2009-04-27 23:59:41 yangtse Exp $
  ***************************************************************************/
 
 /* sws.c: simple (silly?) web server
@@ -41,8 +41,7 @@
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#ifdef _XOPEN_SOURCE_EXTENDED
-/* This define is "almost" required to build on HPUX 11 */
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
 #ifdef HAVE_NETDB_H
@@ -71,14 +70,19 @@
 #define CURL_SWS_FORK_ENABLED
 #endif
 
+#ifdef ENABLE_IPV6
+static bool use_ipv6 = FALSE;
+#endif
+static const char *ipv_inuse = "IPv4";
+
 #define REQBUFSIZ 150000
 #define REQBUFSIZ_TXT "149999"
 
-long prevtestno=-1; /* previous test number we served */
-long prevpartno=-1; /* previous part number we served */
-bool prevbounce=FALSE; /* instructs the server to increase the part number for
-                          a test in case the identical testno+partno request
-                          shows up again */
+static long prevtestno=-1;    /* previous test number we served */
+static long prevpartno=-1;    /* previous part number we served */
+static bool prevbounce=FALSE; /* instructs the server to increase the part
+                                 number for a test in case the identical
+                                 testno+partno request shows up again */
 
 #define RCMD_NORMALREQ 0 /* default request, use the tests file normally */
 #define RCMD_IDLE      1 /* told to sit idle */
@@ -99,13 +103,17 @@ struct httprequest {
   bool ntlm;      /* Authorization ntlm header found */
   int pipe;       /* if non-zero, expect this many requests to do a "piped"
                      request/response */
+  int skip;       /* if non-zero, the server is instructed to not read this
+                     many bytes from a PUT/POST request. Ie the client sends N
+                     bytes said in Content-Length, but the server only reads N
+                     - skip bytes. */
   int rcmd;       /* doing a special command, see defines above */
   int prot_version; /* HTTP version * 10 */
   bool pipelining; /* true if request is pipelined */
 };
 
-int ProcessRequest(struct httprequest *req);
-void storerequest(char *reqbuf, ssize_t totalsize);
+static int ProcessRequest(struct httprequest *req);
+static void storerequest(char *reqbuf, ssize_t totalsize);
 
 #define DEFAULT_PORT 8999
 
@@ -187,7 +195,7 @@ static void sigpipe_handler(int sig)
 }
 #endif
 
-int ProcessRequest(struct httprequest *req)
+static int ProcessRequest(struct httprequest *req)
 {
   char *line=&req->reqbuf[req->checkindex];
   bool chunked = FALSE;
@@ -269,7 +277,7 @@ int ProcessRequest(struct httprequest *req)
         error = ERRNO;
         logmsg("fopen() failed with error: %d %s", error, strerror(error));
         logmsg("Error opening file: %s", filename);
-        logmsg("Couldn't open test file %d", req->testno);
+        logmsg("Couldn't open test file %ld", req->testno);
         req->open = FALSE; /* closes connection */
         return 1; /* done */
       }
@@ -303,6 +311,13 @@ int ProcessRequest(struct httprequest *req)
             req->pipe = num-1; /* decrease by one since we don't count the
                                   first request in this number */
           }
+          else if(1 == sscanf(cmd, "skip: %d", &num)) {
+            logmsg("instructed to skip this number of bytes %d", num);
+            req->skip = num;
+          }
+          else {
+            logmsg("funny instruction found: %s", cmd);
+          }
           free(cmd);
         }
       }
@@ -351,7 +366,7 @@ int ProcessRequest(struct httprequest *req)
        headers, for the pipelining case mostly */
     req->checkindex += (end - line) + strlen(END_OF_HEADERS);
 
-  /* **** Persistancy ****
+  /* **** Persistence ****
    *
    * If the request is a HTTP/1.0 one, we close the connection unconditionally
    * when we're done.
@@ -363,14 +378,17 @@ int ProcessRequest(struct httprequest *req)
    */
 
   do {
-    if(!req->cl && curlx_strnequal("Content-Length:", line, 15)) {
+    if((req->cl==0) && curlx_strnequal("Content-Length:", line, 15)) {
       /* If we don't ignore content-length, we read it and we read the whole
          request including the body before we return. If we've been told to
          ignore the content-length, we will return as soon as all headers
          have been received */
-      req->cl = strtol(line+15, &line, 10);
+      size_t cl = strtol(line+15, &line, 10);
+      req->cl = cl - req->skip;
 
-      logmsg("Found Content-Length: %d in the request", req->cl);
+      logmsg("Found Content-Length: %zu in the request", cl);
+      if(req->skip)
+        logmsg("... but will abort after %zu bytes", req->cl);
       break;
     }
     else if(curlx_strnequal("Transfer-Encoding: chunked", line,
@@ -404,16 +422,16 @@ int ProcessRequest(struct httprequest *req)
        Digest stuff to work in the test suite. */
     req->partno += 1000;
     req->digest = TRUE; /* header found */
-    logmsg("Received Digest request, sending back data %d", req->partno);
+    logmsg("Received Digest request, sending back data %ld", req->partno);
   }
   else if(!req->ntlm &&
           strstr(req->reqbuf, "Authorization: NTLM TlRMTVNTUAAD")) {
     /* If the client is passing this type-3 NTLM header */
     req->partno += 1002;
     req->ntlm = TRUE; /* NTLM found */
-    logmsg("Received NTLM type-3, sending back data %d", req->partno);
+    logmsg("Received NTLM type-3, sending back data %ld", req->partno);
     if(req->cl) {
-      logmsg("  Expecting %d POSTed bytes", req->cl);
+      logmsg("  Expecting %zu POSTed bytes", req->cl);
     }
   }
   else if(!req->ntlm &&
@@ -421,7 +439,14 @@ int ProcessRequest(struct httprequest *req)
     /* If the client is passing this type-1 NTLM header */
     req->partno += 1001;
     req->ntlm = TRUE; /* NTLM found */
-    logmsg("Received NTLM type-1, sending back data %d", req->partno);
+    logmsg("Received NTLM type-1, sending back data %ld", req->partno);
+  }
+  else if((req->partno >= 1000) && strstr(req->reqbuf, "Authorization: Basic")) {
+    /* If the client is passing this Basic-header and the part number is already
+       >=1000, we add 1 to the part number.  This allows simple Basic authentication
+       negotiation to work in the test suite. */
+    req->partno += 1;
+    logmsg("Received Basic request, sending back data %ld", req->partno);
   }
   if(strstr(req->reqbuf, "Connection: close"))
     req->open = FALSE; /* close connection after this request */
@@ -457,7 +482,7 @@ int ProcessRequest(struct httprequest *req)
   if(req->auth_req && !req->auth)
     return 1;
 
-  if(req->cl) {
+  if(req->cl > 0) {
     if(req->cl <= req->offset - (end - req->reqbuf) - strlen(END_OF_HEADERS))
       return 1; /* done */
     else
@@ -468,10 +493,10 @@ int ProcessRequest(struct httprequest *req)
 }
 
 /* store the entire request in a file */
-void storerequest(char *reqbuf, ssize_t totalsize)
+static void storerequest(char *reqbuf, ssize_t totalsize)
 {
   int res;
-  int error;
+  int error = 0;
   ssize_t written;
   ssize_t writeleft;
   FILE *dump;
@@ -482,7 +507,7 @@ void storerequest(char *reqbuf, ssize_t totalsize)
   if (totalsize == 0)
     return;
   else if (totalsize < 0) {
-    logmsg("Invalid size (%d bytes) for request input. Not written to %s",
+    logmsg("Invalid size (%zd bytes) for request input. Not written to %s",
            totalsize, REQUEST_DUMP);
     return;
   }
@@ -508,7 +533,7 @@ void storerequest(char *reqbuf, ssize_t totalsize)
   if (writeleft > 0) {
     logmsg("Error writing file %s error: %d %s",
            REQUEST_DUMP, error, strerror(error));
-    logmsg("Wrote only (%d bytes) of (%d bytes) request input to %s",
+    logmsg("Wrote only (%zd bytes) of (%zd bytes) request input to %s",
            totalsize-writeleft, totalsize, REQUEST_DUMP);
   }
 
@@ -520,7 +545,7 @@ void storerequest(char *reqbuf, ssize_t totalsize)
            REQUEST_DUMP, error, strerror(error));
 
   if(!writeleft)
-    logmsg("Wrote request (%d bytes) input to " REQUEST_DUMP,
+    logmsg("Wrote request (%zd bytes) input to " REQUEST_DUMP,
            totalsize);
 }
 
@@ -531,7 +556,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
   char *reqbuf = req->reqbuf;
   ssize_t got = 0;
 
-  char *pipereq;
+  char *pipereq = NULL;
   int pipereq_length = 0;
 
   if(req->pipelining) {
@@ -539,7 +564,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
     pipereq_length = req->offset - req->checkindex;
   }
 
-  /*** Init the httpreqest structure properly for the upcoming request ***/
+  /*** Init the httprequest structure properly for the upcoming request ***/
 
   req->checkindex = 0;
   req->offset = 0;
@@ -552,6 +577,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
   req->digest = FALSE;
   req->ntlm = FALSE;
   req->pipe = 0;
+  req->skip = 0;
   req->rcmd = RCMD_NORMALREQ;
   req->prot_version = 0;
   req->pipelining = FALSE;
@@ -560,12 +586,19 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
 
   while (req->offset < REQBUFSIZ-1) {
     if(pipereq_length) {
-      memmove(reqbuf, pipereq, pipereq_length); 
+      memmove(reqbuf, pipereq, pipereq_length);
       got = pipereq_length;
       pipereq_length = 0;
     }
-    else
-      got = sread(sock, reqbuf + req->offset, REQBUFSIZ-1 - req->offset);
+    else {
+      if(req->skip)
+        /* we are instructed to not read the entire thing, so we make sure to only
+           read what we're supposed to and NOT read the enire thing the client
+           wants to send! */
+        got = sread(sock, reqbuf + req->offset, req->cl);
+      else
+        got = sread(sock, reqbuf + req->offset, REQBUFSIZ-1 - req->offset);
+    }
     if (got <= 0) {
       if (got < 0) {
         logmsg("recv() returned error: %d", SOCKERRNO);
@@ -579,7 +612,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
       return DOCNUMBER_INTERNAL;
     }
 
-    logmsg("Read %d bytes", got);
+    logmsg("Read %zd bytes", got);
 
     req->offset += got;
     reqbuf[req->offset] = '\0';
@@ -628,14 +661,14 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
   bool persistant = TRUE;
   bool sendfailure = FALSE;
   size_t responsesize;
-  int error;
+  int error = 0;
   int res;
 
   static char weare[256];
 
   char partbuf[80]="data";
 
-  logmsg("Send response number %d part %d", req->testno, req->partno);
+  logmsg("Send response number %ld part %ld", req->testno, req->partno);
 
   switch(req->rcmd) {
   default:
@@ -673,7 +706,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
       logmsg("Identifying ourselves as friends");
       sprintf(msgbuf, "WE ROOLZ: %ld\r\n", (long)getpid());
       msglen = strlen(msgbuf);
-      sprintf(weare, "HTTP/1.1 200 OK\r\nContent-Length: %d\r\n\r\n%s",
+      sprintf(weare, "HTTP/1.1 200 OK\r\nContent-Length: %zu\r\n\r\n%s",
               msglen, msgbuf);
       buffer = weare;
       break;
@@ -773,7 +806,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
       break;
     }
     else {
-      logmsg("Sent off %d bytes", written);
+      logmsg("Sent off %zd bytes", written);
     }
     /* write to file as well */
     fwrite(buffer, 1, written, dump);
@@ -790,7 +823,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
            RESPONSE_DUMP, error, strerror(error));
 
   if(sendfailure) {
-    logmsg("Sending response failed. Only (%d bytes) of (%d bytes) were sent",
+    logmsg("Sending response failed. Only (%zu bytes) of (%zu bytes) were sent",
            responsesize-count, responsesize);
     if(ptr)
       free(ptr);
@@ -799,7 +832,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
     return -1;
   }
 
-  logmsg("Response sent (%d bytes) and written to " RESPONSE_DUMP,
+  logmsg("Response sent (%zu bytes) and written to " RESPONSE_DUMP,
          responsesize);
 
   if(ptr)
@@ -836,7 +869,6 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
   return 0;
 }
 
-bool use_ipv6=FALSE;
 
 int main(int argc, char *argv[])
 {
@@ -878,7 +910,8 @@ int main(int argc, char *argv[])
     }
     else if(!strcmp("--ipv6", argv[arg])) {
 #ifdef ENABLE_IPV6
-      use_ipv6=TRUE;
+      ipv_inuse = "IPv6";
+      use_ipv6 = TRUE;
 #endif
       arg++;
     }
@@ -964,13 +997,7 @@ int main(int argc, char *argv[])
     return 1;
   }
 
-  logmsg("Running IPv%d version on port %d",
-#ifdef ENABLE_IPV6
-         (use_ipv6?6:4)
-#else
-         4
-#endif
-         , port );
+  logmsg("Running %s version on port %d", ipv_inuse, (int)port);
 
   /* start accepting connections */
   rc = listen(sock, 5);
@@ -1049,7 +1076,7 @@ int main(int argc, char *argv[])
       send_doc(msgsock, &req);
 
       if((req.testno < 0) && (req.testno != DOCNUMBER_CONNECT)) {
-        logmsg("special request received, no persistancy");
+        logmsg("special request received, no persistency");
         break;
       }
       if(!req.open) {
