@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: tftpd.c,v 1.35 2008-04-23 23:55:34 yangtse Exp $
+ * $Id: tftpd.c,v 1.53 2009-05-02 02:37:34 yangtse Exp $
  *
  * Trivial file transfer protocol server.
  *
@@ -65,6 +65,9 @@
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
 #ifdef HAVE_ARPA_TFTP_H
 #include <arpa/tftp.h>
 #else
@@ -96,6 +99,11 @@
 /* include memdebug.h last */
 #include "memdebug.h"
 
+#ifdef ENABLE_IPV6
+static bool use_ipv6 = FALSE;
+#endif
+static const char *ipv_inuse = "IPv4";
+
 struct testcase {
   char *buffer;   /* holds the file data to send to the client */
   size_t bufsize; /* size of the data in buffer */
@@ -111,33 +119,46 @@ static struct tftphdr *w_init(void);
 static int readit(struct testcase *test, struct tftphdr **dpp, int convert);
 static int writeit(struct testcase *test, struct tftphdr **dpp, int ct,
                    int convert);
-static void mysignal(int, void (*func)(int));
 
+#define opcode_RRQ   1
+#define opcode_WRQ   2
+#define opcode_DATA  3
+#define opcode_ACK   4
+#define opcode_ERROR 5
 
 #define TIMEOUT         5
 
 #define PKTSIZE SEGSIZE+4
 
-struct formats;
-static int tftp(struct testcase *test, struct tftphdr *tp, int size);
+struct formats {
+  const char *f_mode;
+  int f_convert;
+};
+static struct formats formata[] = {
+  { "netascii",   1 },
+  { "octet",      0 },
+  { NULL,         0 }
+};
+
+static int tftp(struct testcase *test, struct tftphdr *tp, ssize_t size);
 static void nak(int error);
 static void sendtftp(struct testcase *test, struct formats *pf);
 static void recvtftp(struct testcase *test, struct formats *pf);
 static int validate_access(struct testcase *test, const char *, int);
 
 static curl_socket_t peer;
-static int rexmtval = TIMEOUT;
 static int maxtimeout = 5*TIMEOUT;
 
 static char buf[PKTSIZE];
 static char ackbuf[PKTSIZE];
 static struct sockaddr_in from;
-static socklen_t fromlen;
+static curl_socklen_t fromlen;
 
 struct bf {
   int counter;            /* size of data in buffer, or flag */
   char buf[PKTSIZE];      /* room for data packet */
-} bfs[2];
+};
+static struct bf bfs[2];
 
                                 /* Values for bf.counter  */
 #define BF_ALLOC -3             /* alloc'd but not yet filled */
@@ -147,9 +168,9 @@ struct bf {
 static int nextone;     /* index of next buffer to use */
 static int current;     /* index of buffer in use */
 
-                        /* control flags for crlf conversions */
-int newline = 0;        /* fillbuf: in middle of newline expansion */
-int prevchar = -1;      /* putbuf: previous char (cr check) */
+                           /* control flags for crlf conversions */
+static int newline = 0;    /* fillbuf: in middle of newline expansion */
+static int prevchar = -1;  /* putbuf: previous char (cr check) */
 
 static void read_ahead(struct testcase *test,
                        int convert /* if true, convert to ascii */);
@@ -175,7 +196,7 @@ rw_init(int x)              /* init for either read-ahead or write-behind */
    Free it and return next buffer filled with data.
  */
 static int readit(struct testcase *test, struct tftphdr **dpp,
-           int convert /* if true, convert to ascii */)
+                  int convert /* if true, convert to ascii */)
 {
   struct bf *b;
 
@@ -184,7 +205,7 @@ static int readit(struct testcase *test, struct tftphdr **dpp,
 
   b = &bfs[current];              /* look at new buffer */
   if (b->counter == BF_FREE)      /* if it's empty */
-    read_ahead(test, convert);      /* fill it */
+    read_ahead(test, convert);    /* fill it */
 
   *dpp = (struct tftphdr *)b->buf;        /* set caller's ptr */
   return b->counter;
@@ -355,7 +376,7 @@ static int synchnet(curl_socket_t f /* socket to flush */)
   int j = 0;
   char rbuf[PKTSIZE];
   struct sockaddr_in fromaddr;
-  socklen_t fromaddrlen;
+  curl_socklen_t fromaddrlen;
 
   while (1) {
 #if defined(HAVE_IOCTLSOCKET)
@@ -365,9 +386,9 @@ static int synchnet(curl_socket_t f /* socket to flush */)
 #endif
     if (i) {
       j++;
-      fromaddrlen = sizeof fromaddr;
-      (void) recvfrom(f, rbuf, sizeof (rbuf), 0,
-                      (struct sockaddr *)&fromaddr, &fromaddrlen);
+      fromaddrlen = sizeof(fromaddr);
+      (void)recvfrom(f, rbuf, sizeof(rbuf), 0,
+                     (struct sockaddr *)&fromaddr, &fromaddrlen);
     }
     else
       break;
@@ -397,7 +418,6 @@ const char *serverlogfile = DEFAULT_LOGFILE;
 
 #define REQUEST_DUMP  "log/server.input"
 
-char use_ipv6=FALSE;
 
 int main(int argc, char **argv)
 {
@@ -407,7 +427,7 @@ int main(int argc, char **argv)
 #endif /* ENABLE_IPV6 */
 
   struct tftphdr *tp;
-  int n = 0;
+  ssize_t n = 0;
   int arg = 1;
   char *pidname= (char *)".tftpd.pid";
   unsigned short port = DEFAULT_PORT;
@@ -435,7 +455,8 @@ int main(int argc, char **argv)
     }
     else if(!strcmp("--ipv6", argv[arg])) {
 #ifdef ENABLE_IPV6
-      use_ipv6=TRUE;
+      ipv_inuse = "IPv6";
+      use_ipv6 = TRUE;
 #endif
       arg++;
     }
@@ -505,20 +526,14 @@ int main(int argc, char **argv)
     return 1;
   }
 
-  logmsg("Running IPv%d version on port UDP/%d",
-#ifdef ENABLE_IPV6
-         (use_ipv6?6:4)
-#else
-         4
-#endif
-         , port );
+  logmsg("Running %s version on port UDP/%d", ipv_inuse, (int)port);
 
   do {
     fromlen = sizeof(from);
-    n = recvfrom(sock, buf, sizeof (buf), 0,
-                 (struct sockaddr *)&from, &fromlen);
+    n = (ssize_t)recvfrom(sock, buf, sizeof(buf), 0,
+                          (struct sockaddr *)&from, &fromlen);
     if (n < 0) {
-      logmsg("recvfrom:\n");
+      logmsg("recvfrom");
       result = 3;
       break;
     }
@@ -529,13 +544,13 @@ int main(int argc, char **argv)
 
     peer = socket(AF_INET, SOCK_DGRAM, 0);
     if (peer < 0) {
-      logmsg("socket:\n");
+      logmsg("socket");
       result = 2;
       break;
     }
 
     if (connect(peer, (struct sockaddr *)&from, sizeof(from)) < 0) {
-      logmsg("connect: fail\n");
+      logmsg("connect: fail");
       result = 1;
       break;
     }
@@ -543,7 +558,7 @@ int main(int argc, char **argv)
 
     tp = (struct tftphdr *)buf;
     tp->th_opcode = ntohs(tp->th_opcode);
-    if (tp->th_opcode == RRQ || tp->th_opcode == WRQ) {
+    if (tp->th_opcode == opcode_RRQ || tp->th_opcode == opcode_WRQ) {
       memset(&test, 0, sizeof(test));
       if (tftp(&test, tp, n) < 0)
         break;
@@ -554,6 +569,8 @@ int main(int argc, char **argv)
 
     clear_advisor_read_lock(SERVERLOGS_LOCK);
 
+    logmsg("end of one transfer");
+
   } while(1);
 
   clear_advisor_read_lock(SERVERLOGS_LOCK);
@@ -561,19 +578,10 @@ int main(int argc, char **argv)
   return result;
 }
 
-struct formats {
-  const char *f_mode;
-  int f_convert;
-} formats[] = {
-  { "netascii",   1 },
-  { "octet",      0 },
-  { NULL,         0 }
-};
-
 /*
  * Handle initial connection protocol.
  */
-static int tftp(struct testcase *test, struct tftphdr *tp, int size)
+static int tftp(struct testcase *test, struct tftphdr *tp, ssize_t size)
 {
   char *cp;
   int first = 1, ecode;
@@ -623,7 +631,7 @@ again:
   fprintf(server, "mode: %s\n", mode);
   fclose(server);
 
-  for (pf = formats; pf->f_mode; pf++)
+  for (pf = formata; pf->f_mode; pf++)
     if (strcmp(pf->f_mode, mode) == 0)
       break;
   if (!pf->f_mode) {
@@ -635,7 +643,7 @@ again:
     nak(ecode);
     return 1;
   }
-  if (tp->th_opcode == WRQ)
+  if (tp->th_opcode == opcode_WRQ)
     recvtftp(test, pf);
   else
     sendtftp(test, pf);
@@ -731,14 +739,18 @@ static int validate_access(struct testcase *test,
     return EACCESS; /* failure */
   }
 
+  logmsg("file opened and all is good");
   return 0;
 }
 
-int timeout;
+static int timeout;
 #ifdef HAVE_SIGSETJMP
-sigjmp_buf timeoutbuf;
+static sigjmp_buf timeoutbuf;
 #endif
 
+#if defined(HAVE_ALARM) && defined(SIGALRM)
+static int rexmtval = TIMEOUT;
+
 static void timer(int signum)
 {
   (void)signum;
@@ -755,36 +767,43 @@ static void timer(int signum)
 #endif
 }
 
+static void justtimeout(int signum)
+{
+  (void)signum;
+}
+#endif  /* HAVE_ALARM && SIGALRM */
+
+static unsigned short sendblock;
+static struct tftphdr *sdp;
+static struct tftphdr *sap; /* ack packet */
 /*
  * Send the requested file.
  */
 static void sendtftp(struct testcase *test, struct formats *pf)
 {
-  struct tftphdr *dp;
-  struct tftphdr *ap;    /* ack packet */
-  unsigned short block = 1;
   int size;
   ssize_t n;
+  sendblock = 1;
 #if defined(HAVE_ALARM) && defined(SIGALRM)
   mysignal(SIGALRM, timer);
 #endif
-  dp = r_init();
-  ap = (struct tftphdr *)ackbuf;
+  sdp = r_init();
+  sap = (struct tftphdr *)ackbuf;
   do {
-    size = readit(test, &dp, pf->f_convert);
+    size = readit(test, &sdp, pf->f_convert);
     if (size < 0) {
       nak(ERRNO + 100);
       return;
     }
-    dp->th_opcode = htons((u_short)DATA);
-    dp->th_block = htons((u_short)block);
+    sdp->th_opcode = htons((u_short)opcode_DATA);
+    sdp->th_block = htons((u_short)sendblock);
     timeout = 0;
 #ifdef HAVE_SIGSETJMP
     (void) sigsetjmp(timeoutbuf, 1);
 #endif
     send_data:
-    if (swrite(peer, dp, size + 4) != size + 4) {
-      logmsg("write\n");
+    if (swrite(peer, sdp, size + 4) != size + 4) {
+      logmsg("write");
       return;
     }
     read_ahead(test, pf->f_convert);
@@ -797,58 +816,54 @@ static void sendtftp(struct testcase *test, struct formats *pf)
       alarm(0);
 #endif
       if (n < 0) {
-        logmsg("read: fail\n");
+        logmsg("read: fail");
         return;
       }
-      ap->th_opcode = ntohs((u_short)ap->th_opcode);
-      ap->th_block = ntohs((u_short)ap->th_block);
+      sap->th_opcode = ntohs((u_short)sap->th_opcode);
+      sap->th_block = ntohs((u_short)sap->th_block);
 
-      if (ap->th_opcode == ERROR) {
+      if (sap->th_opcode == opcode_ERROR) {
         logmsg("got ERROR");
         return;
       }
 
-      if (ap->th_opcode == ACK) {
-        if (ap->th_block == block) {
+      if (sap->th_opcode == opcode_ACK) {
+        if (sap->th_block == sendblock) {
           break;
         }
         /* Re-synchronize with the other side */
         (void) synchnet(peer);
-        if (ap->th_block == (block -1)) {
+        if (sap->th_block == (sendblock-1)) {
           goto send_data;
         }
       }
 
     }
-    block++;
+    sendblock++;
   } while (size == SEGSIZE);
 }
 
-static void justtimeout(int signum)
-{
-  (void)signum;
-}
-
 
+static unsigned short recvblock;
+static struct tftphdr *rdp;
+static struct tftphdr *rap; /* ack buffer */
 /*
  * Receive a file.
  */
 static void recvtftp(struct testcase *test, struct formats *pf)
 {
-  struct tftphdr *dp;
-  struct tftphdr *ap;    /* ack buffer */
-  unsigned short block = 0;
   ssize_t n, size;
+  recvblock = 0;
 #if defined(HAVE_ALARM) && defined(SIGALRM)
   mysignal(SIGALRM, timer);
 #endif
-  dp = w_init();
-  ap = (struct tftphdr *)ackbuf;
+  rdp = w_init();
+  rap = (struct tftphdr *)ackbuf;
   do {
     timeout = 0;
-    ap->th_opcode = htons((u_short)ACK);
-    ap->th_block = htons((u_short)block);
-    block++;
+    rap->th_opcode = htons((u_short)opcode_ACK);
+    rap->th_block = htons((u_short)recvblock);
+    recvblock++;
 #ifdef HAVE_SIGSETJMP
     (void) sigsetjmp(timeoutbuf, 1);
 #endif
@@ -862,7 +877,7 @@ send_ack:
 #ifdef HAVE_ALARM
       alarm(rexmtval);
 #endif
-      n = sread(peer, dp, PKTSIZE);
+      n = sread(peer, rdp, PKTSIZE);
 #ifdef HAVE_ALARM
       alarm(0);
 #endif
@@ -870,22 +885,22 @@ send_ack:
         logmsg("read: fail\n");
         goto abort;
       }
-      dp->th_opcode = ntohs((u_short)dp->th_opcode);
-      dp->th_block = ntohs((u_short)dp->th_block);
-      if (dp->th_opcode == ERROR)
+      rdp->th_opcode = ntohs((u_short)rdp->th_opcode);
+      rdp->th_block = ntohs((u_short)rdp->th_block);
+      if (rdp->th_opcode == opcode_ERROR)
         goto abort;
-      if (dp->th_opcode == DATA) {
-        if (dp->th_block == block) {
+      if (rdp->th_opcode == opcode_DATA) {
+        if (rdp->th_block == recvblock) {
           break;                         /* normal */
         }
         /* Re-synchronize with the other side */
         (void) synchnet(peer);
-        if (dp->th_block == (block-1))
+        if (rdp->th_block == (recvblock-1))
           goto send_ack;                 /* rexmit */
       }
     }
 
-    size = writeit(test, &dp, (int)(n - 4), pf->f_convert);
+    size = writeit(test, &rdp, (int)(n - 4), pf->f_convert);
     if (size != (n-4)) {                 /* ahem */
       if (size < 0)
         nak(ERRNO + 100);
@@ -896,8 +911,8 @@ send_ack:
   } while (size == SEGSIZE);
   write_behind(test, pf->f_convert);
 
-  ap->th_opcode = htons((u_short)ACK);   /* send the "final" ack */
-  ap->th_block = htons((u_short)(block));
+  rap->th_opcode = htons((u_short)opcode_ACK);  /* send the "final" ack */
+  rap->th_block = htons((u_short)recvblock);
   (void) swrite(peer, ackbuf, 4);
 #if defined(HAVE_ALARM) && defined(SIGALRM)
   mysignal(SIGALRM, justtimeout);        /* just abort read on timeout */
@@ -908,8 +923,8 @@ send_ack:
   alarm(0);
 #endif
   if (n >= 4 &&                          /* if read some data */
-      dp->th_opcode == DATA &&           /* and got a data block */
-      block == dp->th_block) {           /* then my last ack was lost */
+      rdp->th_opcode == opcode_DATA &&   /* and got a data block */
+      recvblock == rdp->th_block) {      /* then my last ack was lost */
     (void) swrite(peer, ackbuf, 4);      /* resend final ack */
   }
 abort:
@@ -919,7 +934,8 @@ abort:
 struct errmsg {
   int e_code;
   const char *e_msg;
-} errmsgs[] = {
+};
+static struct errmsg errmsgs[] = {
   { EUNDEF,       "Undefined error code" },
   { ENOTFOUND,    "File not found" },
   { EACCESS,      "Access violation" },
@@ -942,7 +958,7 @@ static void nak(int error)
   struct errmsg *pe;
 
   tp = (struct tftphdr *)buf;
-  tp->th_opcode = htons((u_short)ERROR);
+  tp->th_opcode = htons((u_short)opcode_ERROR);
   tp->th_code = htons((u_short)error);
   for (pe = errmsgs; pe->e_code >= 0; pe++)
     if (pe->e_code == error)
